@@ -3,7 +3,7 @@
 /* #define PQXX_BROKEN_USING_DECL 1 */
 #define PQXX_HAVE_CHAR_TRAITS 1
 #define PQXX_HAVE_CPP_PRAGMA_MESSAGE 1
-#define PQXX_HAVE_CPP_WARNING 1
+/* #define PQXX_HAVE_CPP_WARNING 1 */
 #define PQXX_HAVE_IOS 1
 #define PQXX_HAVE_LIMITS 1
 #define PQXX_HAVE_LONG_DOUBLE 1
