@@ -7,3 +7,5 @@
 #define PQXX_HAVE_PQRESULTERRORFIELD 1
 #define PQXX_HAVE_PQUNESCAPEBYTEA 1
 #define PQXX_HAVE_PQFTABLE 1
+#define PQXX_HAVE_PQSERVERVERSION 1
+#define PQXX_HAVE_PQPREPARE 1
