@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -15,7 +15,7 @@
  *
  *-------------------------------------------------------------------------
  */
-#include "pqxx/compiler.h"
+#include "pqxx/compiler-internal.hxx"
 
 #include <algorithm>
 #include <cassert>
@@ -28,15 +28,18 @@
 #include <sys/select.h>
 #else
 #include <sys/types.h>
-#if defined(HAVE_UNISTD_H)
-#include <unistd.h>
-#elif defined(_WIN32)
+#if defined(_WIN32)
 #include <winsock2.h>
 #endif
-#endif	// PQXX_HAVE_SYS_SELECT_H
+#if defined(HAVE_UNISTD_H)
+#include <unistd.h>
+#endif
+#endif // PQXX_HAVE_SYS_SELECT_H
 
 #include "libpq-fe.h"
 
+#include "pqxx/binarystring"
+#include "pqxx/connection"
 #include "pqxx/connection_base"
 #include "pqxx/nontransaction"
 #include "pqxx/pipeline"
@@ -45,7 +48,9 @@
 #include "pqxx/trigger"
 
 using namespace PGSTD;
+using namespace pqxx;
 using namespace pqxx::internal;
+using namespace pqxx::prepare;
 
 
 extern "C"
@@ -82,29 +87,27 @@ static void clear_fdmask(fd_set *mask)
 }
 
 
-pqxx::connection_base::connection_base(const string &ConnInfo) :
-  m_ConnInfo(ConnInfo),
+pqxx::connection_base::connection_base(connectionpolicy &pol) :
   m_Conn(0),
+  m_policy(pol),
+  m_Completed(false),
   m_Trans(),
   m_Noticer(),
+  m_defaultNoticeProcessor(0),
   m_Trace(0),
   m_caps(),
   m_inhibit_reactivation(false),
-  m_reactivation_avoidance(0)
+  m_reactivation_avoidance(),
+  m_unique_id(0)
 {
+  clearcaps();
 }
 
 
-pqxx::connection_base::connection_base(const char ConnInfo[]) :
-  m_ConnInfo(ConnInfo ? ConnInfo : ""),
-  m_Conn(0),
-  m_Trans(),
-  m_Noticer(),
-  m_Trace(0),
-  m_caps(),
-  m_inhibit_reactivation(false),
-  m_reactivation_avoidance(0)
+void pqxx::connection_base::init()
 {
+  m_Conn = m_policy.do_startconnect(m_Conn);
+  if (m_policy.is_ready(m_Conn)) activate();
 }
 
 
@@ -114,15 +117,24 @@ int pqxx::connection_base::backendpid() const throw ()
 }
 
 
+namespace
+{
+int socket_of(const ::pqxx::internal::pq::PGconn *c)
+{
+  return c ? PQsocket(c) : -1;
+}
+}
+
+
 int pqxx::connection_base::sock() const throw ()
 {
-  return m_Conn ? PQsocket(m_Conn) : -1;
+  return socket_of(m_Conn);
 }
 
 
 void pqxx::connection_base::activate()
 {
-  if (!is_open())
+  if (!is_open()) try
   {
     if (m_inhibit_reactivation)
       throw broken_connection("Could not reactivate connection; "
@@ -130,41 +142,48 @@ void pqxx::connection_base::activate()
 
     // If any objects were open that didn't survive the closing of our
     // connection, don't try to reactivate
-    if (m_reactivation_avoidance) return;
+    if (m_reactivation_avoidance.get()) return;
 
-    startconnect();
-    completeconnect();
+    m_Conn = m_policy.do_startconnect(m_Conn);
+    m_Conn = m_policy.do_completeconnect(m_Conn);
+    m_Completed = true;	// (But retracted if error is thrown below)
 
-    if (!is_open())
-    {
-      const string Msg( ErrMsg() );
-      disconnect();
-      throw broken_connection(Msg);
-    }
+    if (!is_open()) throw broken_connection();
 
     SetupState();
   }
+  catch (const broken_connection &e)
+  {
+    const string Msg( ErrMsg() );
+    disconnect();
+    m_Completed = false;
+    throw broken_connection(e.what());
+  }
+  catch (const exception &)
+  {
+    m_Completed = false;
+    throw;
+  }
 }
 
 
 void pqxx::connection_base::deactivate()
 {
-  if (m_Conn)
-  {
-    if (m_Trans.get())
-      throw logic_error("Attempt to deactivate connection while " +
-	  		m_Trans.get()->description() + " still open");
-  }
+  if (!m_Conn) return;
 
-  if (m_reactivation_avoidance)
+  if (m_Trans.get())
+    throw logic_error("Attempt to deactivate connection while " +
+	m_Trans.get()->description() + " still open");
+
+  if (m_reactivation_avoidance.get())
   {
     process_notice("Attempt to deactivate connection while it is in a state "
 	"that cannot be fully recovered later (ignoring)");
     return;
   }
 
-  dropconnect();
-  disconnect();
+  m_Completed = false;
+  m_Conn = m_policy.do_disconnect(m_Conn);
 }
 
 
@@ -201,6 +220,12 @@ string pqxx::connection_base::RawGetVar(const string &Var)
 }
 
 
+void pqxx::connection_base::clearcaps() throw ()
+{
+  for (int i=0; i<cap_end; ++i) m_caps[i] = false;
+}
+
+
 /** Set up various parts of logical connection state that may need to be
  * recovered because the physical connection to the database was lost and is
  * being reset, or that may not have been initialized yet.
@@ -213,8 +238,7 @@ void pqxx::connection_base::SetupState()
   if (Status() != CONNECTION_OK)
   {
     const string Msg( ErrMsg() );
-    dropconnect();
-    disconnect();
+    m_Conn = m_policy.do_disconnect(m_Conn);
     throw runtime_error(Msg);
   }
 
@@ -224,27 +248,17 @@ void pqxx::connection_base::SetupState()
   for (PSMap::iterator p = m_prepared.begin(); p != prepared_end; ++p)
     p->second.registered = false;
 
-  if (m_Noticer.get())
-    PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, m_Noticer.get());
+  m_defaultNoticeProcessor = 0;
+  if (m_Noticer.get()) switchnoticer(m_Noticer);
 
   InternalSetTrace();
 
   if (!m_Triggers.empty() || !m_Vars.empty())
   {
-    // Pipeline all queries needed to restore triggers and variables.  Use
-    // existing nontransaction if we're in one, or set up a temporary one.
-    // Note that at this stage, the presence of a transaction object doesn't
-    // mean much.  Even if it is a dbtransaction, no BEGIN has been issued yet
-    // in the new connection.
-    auto_ptr<nontransaction> n;
-    if (!m_Trans.get())
-    {
-      // Jump through a few hoops to satisfy dear old g++ 2.95
-      auto_ptr<nontransaction> p(new nontransaction(*this,"connection_setup"));
-      n = p;
-    }
-    pipeline p(*m_Trans.get(), "restore_state");
-    p.retain(m_Triggers.size() + m_Vars.size());
+    stringstream restore_query;
+
+    // Pipeline all queries needed to restore triggers and variables, so we can
+    // send them over in one go.
 
     // Reinstate all active triggers
     if (!m_Triggers.empty())
@@ -257,7 +271,7 @@ void pqxx::connection_base::SetupState()
         // issue just one LISTEN for each event.
         if (i->first != Last)
         {
-          p.insert("LISTEN \"" + i->first + "\"");
+          restore_query << "LISTEN \"" << i->first << "\"; ";
           Last = i->first;
         }
       }
@@ -265,11 +279,16 @@ void pqxx::connection_base::SetupState()
 
     const map<string,string>::const_iterator var_end(m_Vars.end());
     for (map<string,string>::const_iterator i=m_Vars.begin(); i!=var_end; ++i)
-      p.insert("SET " + i->first + "=" + i->second);
+      restore_query << "SET " << i->first << "=" << i->second << "; ";
 
     // Now do the whole batch at once
-    while (!p.empty()) p.retrieve();
+    PQsendQuery(m_Conn, restore_query.str().c_str());
+    result r;
+    do r = PQgetResult(m_Conn); while (r);
   }
+
+  m_Completed = true;
+  if (!is_open()) throw broken_connection();
 }
 
 
@@ -303,16 +322,6 @@ void pqxx::connection_base::check_result(const result &R, const char Query[])
      * successfully) and the definitely false impression that the connection is
      * still in a workable state.
      *
-     * Tom Lane was reluctant to change this behaviour, saying that it lacks
-     * robustness to give up on the connection "at the first hint of trouble"
-     * when the socket may conceivably still be usable.  I have suggested that
-     * apart from EINTR, EAGAIN and EWOULDBLOCK, which are already handled as
-     * special cases, an error result from a socket probably indicates that the
-     * operating system has decided the error was not likely to be recoverable,
-     * and second-guessing it is not likely to be productive; and that the
-     * choice to abandon a connection or attempt to revive it is probably best
-     * made before a result for the ongoing query is returned.
-     *
      * A particular worry is connection timeout.  One user observed a 15-minute
      * period of inactivity when he pulled out a network cable, after which
      * libpq finally returned a result object containing an error (the error
@@ -321,7 +330,11 @@ void pqxx::connection_base::check_result(const result &R, const char Query[])
      * it in place of the system's explanation of the error, so even attempting
      * to recognize the string is not a reliable workaround) but said the
      * connection was still operational.
+     *
+     * This bug seems to have been fixed in the libpq that shipped with
+     * PostgreSQL 8.1.
      */
+    // TODO: Only do this extra check when using buggy libpq!
     if (!consume_input()) throw broken_connection(e.what());
     const int fd = sock();
     if (fd < 0) throw broken_connection(e.what());
@@ -359,19 +372,25 @@ void pqxx::connection_base::check_result(const result &R, const char Query[])
 
 void pqxx::connection_base::disconnect() throw ()
 {
-  if (m_Conn)
-  {
-    PQfinish(m_Conn);
-    m_Conn = 0;
-    // When we activate again, the server may be different!
-    for (int i=0; i<cap_end; ++i) m_caps[i] = false;
-  }
+  // When we activate again, the server may be different!
+  clearcaps();
+
+  m_Conn = m_policy.do_disconnect(m_Conn);
 }
 
 
 bool pqxx::connection_base::is_open() const throw ()
 {
-  return m_Conn && (Status() == CONNECTION_OK);
+  return m_Conn && m_Completed && (Status() == CONNECTION_OK);
+}
+
+
+void pqxx::connection_base::switchnoticer(const auto_ptr<noticer> &N) throw ()
+{
+  const PQnoticeProcessor old =
+	PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, N.get());
+  if (!m_defaultNoticeProcessor)
+    m_defaultNoticeProcessor = old;
 }
 
 
@@ -380,8 +399,8 @@ pqxx::connection_base::set_noticer(auto_ptr<noticer> N) throw ()
 {
   if (m_Conn)
   {
-    if (N.get()) PQsetNoticeProcessor(m_Conn, pqxxNoticeCaller, N.get());
-    else PQsetNoticeProcessor(m_Conn, 0, 0);
+    if (N.get()) switchnoticer(N);
+    else PQsetNoticeProcessor(m_Conn, m_defaultNoticeProcessor, 0);
   }
 
   auto_ptr<noticer> Old = m_Noticer;
@@ -663,20 +682,26 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 }
 
 
-void pqxx::connection_base::pq_prepare(const string &name, 
-    const string &def,
-    const string &params)
+pqxx::prepare::declaration pqxx::connection_base::prepare(const string &name,
+    const string &definition)
 {
   PSMap::iterator i = m_prepared.find(name);
   if (i != m_prepared.end())
   {
-    // Quietly accept identical redefinitions
-    if (def == i->second.definition && params == i->second.parameters) return;
-    // Reject non-identical redefinitions
-    throw logic_error("Incompatible redefinition of prepared statement "+name);
-  }
+    if (definition != i->second.definition)
+      throw invalid_argument("Inconsistent redefinition "
+	  "of prepared statement " + name);
 
-  m_prepared.insert(PSMap::value_type(name, prepared_def(def, params)));
+    // Prepare for repeated repetition of parameters
+    i->second.parameters.clear();
+    i->second.complete = false;
+  }
+  else
+  {
+    m_prepared.insert(make_pair(name,
+	  prepare::internal::prepared_def(definition)));
+  }
+  return prepare::declaration(*this,name);
 }
 
 
@@ -687,48 +712,158 @@ void pqxx::connection_base::unprepare(const string &name)
   // Quietly ignore duplicated or spurious unprepare()s
   if (i == m_prepared.end()) return;
 
-  if (i->second.registered) Exec(("DEALLOCATE " + name).c_str(), 0);
+  if (i->second.registered) Exec(("DEALLOCATE \"" + name + "\"").c_str(), 0);
 
   m_prepared.erase(i);
 }
 
 
-pqxx::result pqxx::connection_base::pq_exec_prepared(const string &pname,
-	int nparams,
-	const char *const *params)
+#ifndef PQXX_HAVE_PQEXECPREPARED
+namespace
+{
+string escape_param(const char in[], prepare::param_treatment treatment)
+{
+  if (!in) return "null";
+  
+  switch (treatment)
+  {
+  case treat_binary:
+    return "'" + escape_binary(in) + "'";
+
+  case treat_string:
+    return "'" + pqxx::internal::escape_string(in, strlen(in)) + "'";
+
+  case treat_bool:
+    switch (in[0])
+    {
+    case 't':
+    case 'T':
+    case 'f':
+    case 'F':
+      break;
+    default:
+      {
+        // Looks like a numeric value may have been passed.  Try to convert it
+        // back to a number, then to bool to normalize its representation
+        bool b;
+	from_string(in,b);
+        return to_string(b);
+      }
+    }
+    break;
+
+  case treat_direct:
+    break;
+
+  default:
+    throw logic_error("Unknown treatment for prepared-statement parameter");
+  }
+
+  return in;
+}
+} // namespace
+#endif
+
+
+pqxx::prepare::internal::prepared_def &
+pqxx::connection_base::find_prepared(const string &statement)
+{
+  PSMap::iterator s = m_prepared.find(statement);
+  if (s == m_prepared.end())
+    throw invalid_argument("Unknown prepared statement '" + statement + "'");
+  return s->second;
+}
+
+void pqxx::connection_base::prepare_param_declare(const string &statement,
+    const string &sqltype,
+    param_treatment treatment)
+{
+  prepare::internal::prepared_def &s = find_prepared(statement);
+  if (s.complete)
+    throw logic_error("Attempt to add parameter to prepared statement " +
+	statement +
+	" after its definition was completed");
+  s.addparam(sqltype,treatment);
+}
+
+
+pqxx::result pqxx::connection_base::prepared_exec(const string &statement,
+	const char *const params[],
+	int nparams)
 {
   activate();
 
-  PSMap::iterator p = m_prepared.find(pname);
-  if (p == m_prepared.end())
-    throw logic_error("Unknown prepared statement: " + pname);
+  prepare::internal::prepared_def &s = find_prepared(statement);
+
+  if (nparams != int(s.parameters.size()))
+    throw logic_error("Wrong number of parameters for prepared statement " +
+	statement + ": "
+	"expected " + to_string(s.parameters.size()) + ", "
+	"received " + to_string(nparams));
+
+  s.complete = true;
 
   // "Register" (i.e., define) prepared statement with backend on demand
-  if (!p->second.registered)
+  if (!s.registered && supports(cap_prepared_statements))
   {
-    // TODO: Wrap in nested transaction if backend supports it!
+#ifdef PQXX_HAVE_PQPREPARE
+    PQprepare(m_Conn, statement.c_str(), s.definition.c_str(), 0, 0);
+#else
     stringstream P;
-    P << "PREPARE " << pname << ' ' << p->second.parameters 
-      << " AS " << p->second.definition;
+    P << "PREPARE \"" << statement << '"';
+
+    if (!s.parameters.empty())
+      P << '('
+	<< separated_list(",",
+	   	s.parameters.begin(),
+		s.parameters.end(),
+		prepare::internal::get_sqltype())
+	<< ')';
+
+    P << " AS " << s.definition;
     Exec(P.str().c_str(), 0);
-    p->second.registered = true;
+#endif
+    s.registered = true;
   }
 
 #ifdef PQXX_HAVE_PQEXECPREPARED
-  result R(PQexecPrepared(m_Conn, pname.c_str(), nparams, params, 0, 0, 0));
-
-  check_result(R, pname.c_str());
-
-  get_notifs();
-  return R;
+  result r(PQexecPrepared(m_Conn,statement.c_str(),nparams,params,0,0,0));
 #else
   stringstream Q;
-  Q << "EXECUTE "
-    << pname
-    << ' '
-    << separated_list(",", params, params+nparams);
-  return Exec(Q.str().c_str(), 0);
+  if (supports(cap_prepared_statements))
+  {
+    Q << "EXECUTE \"" << statement << '"';
+    if (nparams)
+    {
+      Q << " (";
+      for (int a = 0; a < nparams; ++a)
+      {
+	Q << escape_param(params[a],s.parameters[a].treatment);
+	if (a < nparams-1) Q << ',';
+      }
+      Q << ')';
+    }
+  }
+  else
+  {
+    // This backend doesn't support prepared statements.  Do our own variable
+    // substitution.
+    string S = s.definition;
+    for (int n = nparams-1; n >= 0; --n)
+    {
+      const string key = "$" + to_string(n+1),
+	           val = escape_param(params[n],s.parameters[n].treatment);
+      const string::size_type keysz = key.size();
+      for (string::size_type h=S.find(key); h!=string::npos; h=S.find(key))
+	S.replace(h,keysz,val);
+    }
+    Q << S;
+  }
+  result r(Exec(Q.str().c_str(), 0));
 #endif
+  check_result(r, statement.c_str());
+  get_notifs();
+  return r;
 }
 
 
@@ -737,10 +872,12 @@ void pqxx::connection_base::Reset()
   if (m_inhibit_reactivation)
     throw broken_connection("Could not reset connection: "
 	"reactivation is inhibited");
-  if (m_reactivation_avoidance) return;
+  if (m_reactivation_avoidance.get()) return;
 
+  // TODO: Probably need to go through a full disconnect/reconnect!
   // Forget about any previously ongoing connection attempts
-  dropconnect();
+  m_Conn = m_policy.do_dropconnect(m_Conn);
+  m_Completed = false;
 
   if (m_Conn)
   {
@@ -758,12 +895,13 @@ void pqxx::connection_base::Reset()
 
 void pqxx::connection_base::close() throw ()
 {
+  m_Completed = false;
 #ifdef PQXX_QUIET_DESTRUCTORS
   auto_ptr<noticer> n(new nonnoticer());
   set_noticer(n);
 #endif
   inhibit_reactivation(false);
-  m_reactivation_avoidance = 0;
+  m_reactivation_avoidance.clear();
   try
   {
     if (m_Trans.get())
@@ -776,7 +914,7 @@ void pqxx::connection_base::close() throw ()
       m_Triggers.clear();
     }
 
-    disconnect();
+    m_Conn = m_policy.do_disconnect(m_Conn);
   }
   catch (...)
   {
@@ -996,6 +1134,57 @@ pqxx::internal::pq::PGresult *pqxx::connection_base::get_result()
 }
 
 
+string pqxx::connection_base::esc(const char str[], size_t maxlen)
+{
+#ifdef PQXX_HAVE_PQESCAPESTRINGCONN
+  // We need a connection object...  This is the one reason why this function is
+  // not const!
+  if (!m_Conn) activate();
+
+  char *const buf = new char[2*maxlen+1];
+  string escaped;
+  try
+  {
+    int err = 0;
+    PQescapeStringConn(m_Conn, buf, str, maxlen, &err);
+    if (err) throw invalid_argument(ErrMsg());
+    escaped = string(buf);
+  }
+  catch (const exception &)
+  {
+    delete [] buf;
+    throw;
+  }
+  delete [] buf;
+
+  return escaped;
+#else
+  return internal::escape_string(str, maxlen);
+#endif
+}
+
+
+string pqxx::connection_base::esc_raw(const unsigned char str[], size_t len)
+{
+  size_t bytes = 0;
+#ifdef PQXX_HAVE_PQESCAPEBYTEACONN
+  // We need a connection object...  This is the one reason why this function is
+  // not const!
+  if (!m_Conn) activate();
+
+  PQAlloc<unsigned char> buf( PQescapeByteaConn(m_Conn, str, len, &bytes) );
+  if (!buf.c_ptr())
+  {
+    // TODO: Distinguish different errors in exception type
+    throw runtime_error(ErrMsg());
+  }
+#else
+  PQAlloc<unsigned char> buf( PQescapeBytea(str, len, &bytes) );
+  if (!buf.c_ptr()) throw bad_alloc();
+#endif
+  return string(reinterpret_cast<char *>(buf.c_ptr()));
+}
+
 
 namespace
 {
@@ -1009,25 +1198,45 @@ void wait_fd(int fd, bool forwrite=false, timeval *tv=0)
 }
 } // namespace
 
+void pqxx::internal::wait_read(const pq::PGconn *c)
+{
+  wait_fd(socket_of(c));
+}
+
+
+void pqxx::internal::wait_read(const pq::PGconn *c,
+    long seconds,
+    long microseconds)
+{
+  timeval tv = { seconds, microseconds };
+  wait_fd(socket_of(c), false, &tv);
+}
+
+
+void pqxx::internal::wait_write(const pq::PGconn *c)
+{
+  wait_fd(socket_of(c), true);
+}
+
 
 void pqxx::connection_base::wait_read() const
 {
-  wait_fd(sock());
+  internal::wait_read(m_Conn);
 }
 
 
 void pqxx::connection_base::wait_read(long seconds, long microseconds) const
 {
-  timeval tv = { seconds, microseconds };
-  wait_fd(sock(), false, &tv);
+  internal::wait_read(m_Conn, seconds, microseconds);
 }
 
 
 void pqxx::connection_base::wait_write() const
 {
-  wait_fd(sock(), true);
+  internal::wait_write(m_Conn);
 }
 
+
 int pqxx::connection_base::await_notification()
 {
   activate();
@@ -1061,28 +1270,23 @@ void pqxx::connection_base::read_capabilities() throw ()
   if (m_Conn) v = PQserverVersion(m_Conn);
 #endif
 
-  m_caps[cap_create_table_with_oids] = (v >= 80000);
+  m_caps[cap_prepared_statements] = (v >= 70300);
   m_caps[cap_cursor_scroll] = (v >= 70400);
-
-  // TODO: Find out which backend versions support these
-  //m_caps[cap_cursor_with_hold] = ();
-  //m_caps[cap_cursor_update] = ();
+  m_caps[cap_cursor_with_hold] = (v >= 70400);
+  m_caps[cap_nested_transactions] = (v >= 80000);
+  m_caps[cap_create_table_with_oids] = (v >= 80000);
 }
 
 
-// TODO: Inline this, once assertions are known to be good
-void pqxx::connection_base::reactivation_avoidance_add(int n) throw ()
+void pqxx::connection_base::set_capability(capability c) throw ()
 {
-  if (is_open()) m_reactivation_avoidance += n;
-  if (m_reactivation_avoidance < 0)
-    throw internal_error("Negative reactivation avoidance on connection");
+  m_caps[c] = true;
 }
 
-// TODO: Inline this, once assertions are known to be good
-void pqxx::connection_base::reactivation_avoidance_dec() throw ()
+
+string pqxx::connection_base::adorn_name(const string &n)
 {
-  if (m_reactivation_avoidance <= 0)
-    throw internal_error("Reactivation avoidance falling below zero");
-  --m_reactivation_avoidance;
+  const string id = to_string(++m_unique_id);
+  return n.empty() ? ("x"+id) : (n+"_"+id);
 }
 
