@@ -15,7 +15,7 @@
  *
  *-------------------------------------------------------------------------
  */
-#include "pqxx/compiler.h"
+#include "pqxx/compiler-internal.hxx"
 
 #include <cerrno>
 #include <stdexcept>
@@ -202,20 +202,23 @@ pqxx::largeobjectaccess::seek(size_type dest, seekdir dir)
 }
 
 
-long pqxx::largeobjectaccess::cseek(off_type dest, seekdir dir) throw ()
+pqxx::largeobjectaccess::pos_type
+pqxx::largeobjectaccess::cseek(off_type dest, seekdir dir) throw ()
 {
   return lo_lseek(RawConnection(), m_fd, dest, StdDirToPQDir(dir));
 }
 
 
-long pqxx::largeobjectaccess::cwrite(const char Buf[], size_type Len) throw ()
+pqxx::largeobjectaccess::pos_type
+pqxx::largeobjectaccess::cwrite(const char Buf[], size_type Len) throw ()
 {
   return
     PGSTD::max(lo_write(RawConnection(),m_fd,const_cast<char *>(Buf), Len), -1);
 }
 
 
-long pqxx::largeobjectaccess::cread(char Buf[], size_type Bytes) throw ()
+pqxx::largeobjectaccess::pos_type
+pqxx::largeobjectaccess::cread(char Buf[], size_type Bytes) throw ()
 {
   return PGSTD::max(lo_read(RawConnection(), m_fd, Buf, Bytes), -1);
 }
