@@ -15,7 +15,7 @@
  *
  *-------------------------------------------------------------------------
  */
-#include "pqxx/compiler.h"
+#include "pqxx/compiler-internal.hxx"
 
 #include "pqxx/dbtransaction"
 #include "pqxx/pipeline"
@@ -34,8 +34,9 @@ const string theDummyQuery("SELECT " + theDummyValue + theSeparator);
 }
 
 
-pqxx::pipeline::pipeline(transaction_base &t, const string &PName) :
-  internal::transactionfocus(t, PName, "pipeline"),
+pqxx::pipeline::pipeline(transaction_base &t, const string &Name) :
+  namedclass("pipeline", Name),
+  transactionfocus(t),
   m_queries(),
   m_issuedrange(),
   m_retain(0),
@@ -175,6 +176,8 @@ void pqxx::pipeline::issue()
   pqxxassert(!m_dummy_pending);
   pqxxassert(m_num_waiting);
 
+  // TODO: Wrap in nested transaction if available, for extra "replayability"
+
   // Retrieve that NULL result for the last query, if needed
   obtain_result();
 
@@ -316,7 +319,7 @@ void pqxx::pipeline::obtain_dummy()
     }
     while (m_issuedrange.first != stop);
   }
-  catch (const exception &e)
+  catch (const exception &)
   {
     pqxxassert(m_issuedrange.first != m_queries.end());
 
