@@ -8,7 +8,7 @@
  *   pqxx::transaction_base defines the interface for any abstract class that
  *   represents a database transaction
  *
- * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -16,7 +16,7 @@
  *
  *-------------------------------------------------------------------------
  */
-#include "pqxx/compiler.h"
+#include "pqxx/compiler-internal.hxx"
 
 #include <stdexcept>
 
@@ -30,20 +30,20 @@ using namespace PGSTD;
 using namespace pqxx::internal;
 
 
-pqxx::transaction_base::transaction_base(connection_base &C,
-    					 const string &TName,
-					 const string &CName) :
-  namedclass(TName, CName),
+pqxx::transaction_base::transaction_base(connection_base &C, bool direct) :
+  namedclass("transaction_base"),
   m_Conn(C),
-  m_UniqueCursorNum(1),
   m_Focus(),
   m_Status(st_nascent),
   m_Registered(false),
   m_PendingError(),
-  m_reactivation_avoidance(0)
+  m_reactivation_avoidance()
 {
-  m_Conn.RegisterTransaction(this);
-  m_Registered = true;
+  if (direct)
+  {
+    m_Conn.RegisterTransaction(this);
+    m_Registered = true;
+  }
 }
 
 
@@ -108,7 +108,7 @@ void pqxx::transaction_base::commit()
     // Transaction may or may not have been committed.  Report the problem but
     // don't compound our troubles by throwing.
     throw logic_error(description() +
-		      "committed again while in an undetermined state\n");
+		      "committed again while in an indeterminate state");
 
   default:
     throw internal_error("pqxx::transaction: invalid status code");
@@ -189,6 +189,25 @@ void pqxx::transaction_base::abort()
 }
 
 
+string pqxx::transaction_base::esc(const char str[]) const
+{
+  return m_Conn.esc(str, strlen(str));
+}
+
+
+string pqxx::transaction_base::esc(const string &str) const
+{
+  return m_Conn.esc(str.c_str(), str.size());
+}
+
+
+string pqxx::transaction_base::esc_raw(const string &str) const
+{
+  const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
+  return m_Conn.esc_raw(p, str.size());
+}
+
+
 pqxx::result pqxx::transaction_base::exec(const char Query[],
     					  const string &Desc)
 {
@@ -213,17 +232,11 @@ pqxx::result pqxx::transaction_base::exec(const char Query[],
     break;
 
   case st_committed:
-    throw logic_error("Attempt to execute query " + N +
-	              "in committed " + description());
-
   case st_aborted:
-    throw logic_error("Attempt to execute query " + N +
-	              "in aborted " + description());
-
   case st_in_doubt:
-    throw logic_error("Attempt to execute query " + N + "in " +
-		      description() + ", "
-		      "which is in indeterminate state");
+    throw logic_error("Attempt to execute query " + N + " "
+	"in " + description() + ", which is already closed");
+
   default:
     throw internal_error("pqxx::transaction: invalid status code");
   }
@@ -233,6 +246,19 @@ pqxx::result pqxx::transaction_base::exec(const char Query[],
 }
 
 
+pqxx::prepare::invocation
+pqxx::transaction_base::prepared(const string &statement)
+{
+  return prepare::invocation(*this, statement);
+}
+
+
+pqxx::result pqxx::transaction_base::prepared_exec(const string &statement,
+    const char *const params[],
+    int nparams)
+{
+  return m_Conn.prepared_exec(statement, params, nparams);
+}
 
 
 void pqxx::transaction_base::set_variable(const string &Var,
@@ -277,38 +303,37 @@ void pqxx::transaction_base::Begin()
 
 void pqxx::transaction_base::End() throw ()
 {
-  if (!m_Registered) return;
-
   try
   {
-    m_Conn.UnregisterTransaction(this);
-    m_Registered = false;
+    try { CheckPendingError(); }
+    catch (const exception &e) { m_Conn.process_notice(e.what()); }
 
-    CheckPendingError();
+    if (m_Registered)
+    {
+      m_Registered = false;
+      m_Conn.UnregisterTransaction(this);
+    }
+
+    if (m_Status != st_active) return;
 
     if (m_Focus.get())
       m_Conn.process_notice("Closing " + description() + " "
 			    " with " + m_Focus.get()->description() + " "
 			    "still open\n");
 
-    if (m_Status == st_active) abort();
+    try { abort(); }
+    catch (const exception &e) { m_Conn.process_notice(e.what()); }
 
-    if (m_reactivation_avoidance)
+    const int ra = m_reactivation_avoidance.get();
+    if (ra)
     {
-      m_Conn.reactivation_avoidance_add(m_reactivation_avoidance);
       reactivation_avoidance_clear();
+      conn().m_reactivation_avoidance.add(ra);
     }
   }
   catch (const exception &e)
   {
-    try
-    {
-      m_Conn.process_notice(string(e.what()) + "\n");
-    }
-    catch (const exception &)
-    {
-      m_Conn.process_notice(e.what());
-    }
+    try { m_Conn.process_notice(e.what()); } catch (const exception &) {}
   }
 }
 
