@@ -1,6 +1,10 @@
 // We need some libpqxx-internal configuration items.  DON'T INCLUDE THIS HEADER
 // IN NORMAL CLIENT PROGRAMS!
-#include "pqxx/compiler.h"
+#include "pqxx/compiler-internal.hxx"
+
+#ifdef PQXX_HAVE_LOCALE
+#include <locale>
+#endif
 
 #include <iostream>
 
@@ -36,27 +40,15 @@ void check(string ref, string val, string vdesc)
 	"<> " + ref + "'");
 }
 
-void esc(string str, string expected=string())
-{
-  if (expected.empty()) expected = str;
-  check(expected, sqlesc(str), "string");
-
-  if (str.size() < strlen(str.c_str()))
-  {
-    check(expected, sqlesc(str.c_str()), "const char[]");
-    check(expected, sqlesc(str.c_str(), str.size()), "const char[],size_t");
-    check(expected, sqlesc(str.c_str(),
-	  strlen(str.c_str())), "const char[],strlen(...)");
-    check(expected, sqlesc(str.c_str(),10000), "const char[],1000");
-  }
-}
-
 template<typename T> inline void strconv(string type,
     const T &Obj,
     string expected)
 {
   const string Objstr(to_string(Obj));
 
+  cout << '\t' << type << '\t' << ":\t" << Objstr
+       << " (expect " << expected << ")" << endl;
+
   check(expected, Objstr, type);
   T NewObj;
   from_string(Objstr, NewObj);
@@ -119,19 +111,9 @@ int main()
     vector<int> V2(I2);
     testitems(items<int>(V2),2);
 
-    const char weird[] = "foo\t\0bar";
+    const char weird[] = "foo\t\n\0bar";
     const string weirdstr(weird, sizeof(weird)-1);
 
-    cout << "Testing SQL string escape functions..." << endl;
-    esc("");
-    esc("foo");
-    esc("foo bar");
-    esc("unquote' ha!", "unquote'' ha!");
-    esc("'", "''");
-    esc("\\", "\\\\");
-    esc("\t");
-    esc(weirdstr, "foo\t\\000bar");
-
     cout << "Testing string conversions..." << endl;
     strconv("const char[]", "", "");
     strconv("const char[]", "foo", "foo");
@@ -145,9 +127,17 @@ int main()
 #else
     const long long_min = LONG_MIN, long_max = LONG_MAX;
 #endif
+    stringstream lminstr, lmaxstr;
+#if defined(PQXX_HAVE_IMBUE)
+    lminstr.imbue(locale("C"));
+    lmaxstr.imbue(locale("C"));
+#endif
+    lminstr << long_min;
+    lmaxstr << long_max;
 
-    strconv("long", long_min, to_string(long_min));
-    strconv("long", long_max, to_string(long_max));
+    strconv("long", 0, "0");
+    strconv("long", long_min, lminstr.str());
+    strconv("long", long_max, lmaxstr.str());
     strconv("double", 0.0, "0");
     strconv("double", not_a_number, "nan");
     strconv("string", string(), "");
