@@ -82,17 +82,17 @@ public:
     throw runtime_error("Transaction deliberately aborted");
   }
 
-  void OnAbort(const char Reason[]) throw ()
+  void on_abort(const char Reason[]) throw ()
   {
     cout << "(Expected) Transactor " << Name() << " failed: " << Reason << endl;
   }
 
-  void OnCommit()
+  void on_commit()
   {
     cerr << "Transactor " << Name() << " succeeded." << endl;
   }
 
-  void OnDoubt() throw ()
+  void on_doubt() throw ()
   {
     cerr << "Transactor " << Name() << " in indeterminate state!" << endl;
   }
