@@ -59,7 +59,7 @@ public:
     T.exec("DELETE FROM " + m_Table);
   }
 
-  void OnCommit()
+  void on_commit()
   {
     cout << "Table successfully cleared." << endl;
   }
@@ -106,7 +106,7 @@ public:
     CheckState(Org);
   }
 
-  void OnCommit()
+  void on_commit()
   {
     cout << "Table successfully copied." << endl;
   }
