@@ -63,7 +63,7 @@ public:
     T.exec("NOTIFY \"" + m_Trigger + "\"");
   }
 
-  void OnAbort(const char Reason[]) throw ()
+  void on_abort(const char Reason[]) throw ()
   {
     try
     {
