@@ -6,12 +6,16 @@
 
 #if defined(PQXX_HAVE_SYS_SELECT_H)
 #include <sys/select.h>
-#elif defined(PQXX_HAVE_UNISTD_H)
+#else
 #include <sys/types.h>
+#if defined(PQXX_HAVE_UNISTD_H)
 #include <unistd.h>
-#elif defined(_WIN32)
+#endif
+#if defined(_WIN32)
+#define NOMINMAX
 #include <winsock2.h>
 #endif
+#endif // PQXX_HAVE_SYS_SELECT_H
 
 #include <pqxx/connection>
 #include <pqxx/nontransaction>
@@ -76,7 +80,7 @@ public:
     T.exec("NOTIFY \"" + m_Trigger + "\"");
   }
 
-  void OnAbort(const char Reason[]) throw ()
+  void on_abort(const char Reason[]) throw ()
   {
     try
     {
